﻿using System;
using System.Windows;
using System.IO;
using System.ComponentModel;
using System.Threading;
using System.Diagnostics;
using INI;

namespace TravelBuddy
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>

	public partial class MainWindow : Window , IDisposable
	{
		//Polling rate to use for data text file
		private const int cPOLLRATE = 1000;
		//Create a separate thread for updating the GUI
		private readonly BackgroundWorker bw = new BackgroundWorker();
		//Create string for storing savedata path
		private string strSaveDataPath = "";
		//Create array to store data from text file
		private string[] aryTextValues = new string[100];
		//Create thread flag
		public bool boolRunning = false;
		//Create a stopwatch to keep track of elapsed time
		private Stopwatch swRF = new Stopwatch();
		//Create flag for refuelling detected & the type occuring
		private sbyte sbytStartRF = 0;
		//Create flag for Text Line display type
		private sbyte sbytTxtLineDisplay = 0;
		//Create variable to store the last fuel on board value
		private int intLstFOB = -1;
		//Create array of integers for storing last range and the sector from which it was reported (Range,SX,SY,SZ)
		private int[] intLstRange = new int[4];
		private int intLstPwr = 0;

		public MainWindow()
		{
			InitializeComponent();
			ReadINIFile();
			intLstRange[0] = -1;
			//Define the handlers for the functions required by the separate thread
			bw.DoWork += bw_DoWork;
			bw.RunWorkerCompleted += bw_RunWorkerCompleted;
			bw.WorkerSupportsCancellation = true;
			//Clear out the output, I'm too lazy to do this in the designer
			this.txtLine1.Text = "";
			this.txtLine2.Text = "";
			this.txtLine3.Text = "";
			this.txtLine4.Text = "";
			this.txtLine5.Text = "";
			this.txtLine6.Text = "";
			this.txtLine6.Foreground = System.Windows.Media.Brushes.Cyan;
		}

		//This is the function that executes when the background worker thread is called (asynchronous)
		void bw_DoWork(object sender, DoWorkEventArgs e)
		{
			Thread.Sleep(cPOLLRATE);
			if (boolRunning == true)
			{
				//Define the sender as a return to the caller
				BackgroundWorker bw = (BackgroundWorker)sender;
				//Declare a new file read process
				StreamReader sr = new StreamReader(strSaveDataPath);
				//Declare an index for the array to increment during file read
				int intIndex = 0;
				//Use PEEK to check for end of file, read each line and write it to the array, trimming white space from the ends
				while (sr.Peek() != -1)
				{
					aryTextValues[intIndex] = sr.ReadLine().Trim();
					intIndex++;
				}
				//Close the file
				sr.Close();
			}
			else
			{
				bw.CancelAsync();
			}
		}

		//This is the function that executes when the background worker thread reaches its end
		void bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (boolRunning == true)
			{
				//Initialize variables for the calculations
				int intRange = 0;
				int intDriveRange = 0;

				double dblPeriod = 0;
				double dblETF = 0;
				double dblFillRate = 0;
				double dblERR = 0;
				int intFOB = Convert.ToInt32(aryTextValues[1].Substring(0, aryTextValues[1].IndexOf('/')));
				int intTank = Convert.ToInt32(aryTextValues[1].Substring(aryTextValues[1].IndexOf('/') + 1));
				//Create a flag for if fuel converter equiped
				bool boolFuelConverter = false;
				//Create a flag for remaining number of sectors - if we start and there are no more than 1 sector, skip the calculations
				bool boolMoreThan1Sector = false;

				//Check for Inertia control - line 84 - 0 base array so index is 83
				if (aryTextValues[83] == "1")
				{
					//Set the foreground color of the text block
					this.txtLine1.Foreground = System.Windows.Media.Brushes.Cyan;
					//Set the text of the text block
					this.txtLine1.Text = "Inertial Mode Activated";
				}
				else
				{
					if (sbytStartRF == 0)
						this.txtLine1.Foreground = System.Windows.Media.Brushes.Red;
					else
						this.txtLine1.Foreground = System.Windows.Media.Brushes.Cyan;
					this.txtLine1.Text = "Warning: IDS Engaged!";
				}

				//Check for energy settings - line 82 - 0 base array so index is 81
				if (aryTextValues[81] == "-5S/5W")
				{
					//Set the foreground color of the text block
					this.txtLine2.Foreground = System.Windows.Media.Brushes.Cyan;
					//Set the text of the text block
					this.txtLine2.Text = "Energy Bias Set: -5S/5W";
				}
				else
				{
					//Set the foreground color of the text block
					this.txtLine2.Foreground = System.Windows.Media.Brushes.Red;
					//Set the text of the text block
					this.txtLine2.Text = "Warning: Energy Bias Not -5S/5W!";
				}
				//Set energy recharge period from bias setting
				dblPeriod = Convert.ToDouble(aryTextValues[81].Substring(aryTextValues[81].IndexOf('/') + 1, aryTextValues[81].Length - aryTextValues[81].IndexOf('/') - 2));
				dblPeriod = 120 / (10 + dblPeriod);

				//Check equipment in lines 51-58 - 0 base array so indices 50-57
				for (int x = 50; x < 58; x++)
				{
					//Use a case structure to check for the following equipment, assign milage and engine values for drives, & set flag values
					switch (aryTextValues[x])
					{
						case "Mantis Jump Drive":
							intDriveRange = 10;
							break;
						case "Fulcrum Drive C5":
							intDriveRange = 5;
							break;
						case "Fulcrum Drive C4":
							intDriveRange = 4;
							break;
						case "Fulcrum Drive C3":
							intDriveRange = 3;
							break;
						case "Fulcrum Drive C2":
							intDriveRange = 2;
							break;
						case "Fulcrum Drive C1":
							intDriveRange = 1;
							break;
						case "Fuel Converter":
							boolFuelConverter = true;
							break;
						default:
							//No drive found, do nothing - this prevents us from overwriting a "found" status when we're reading a line after the drive we found
							break;
					}
				}

				//Check if fuel has gone up indicating refuelling and detemine the source(s)
				if (intLstFOB > -1 && intFOB > intLstFOB && intFOB < intLstFOB + 50)
				{
					if (sbytStartRF < 2 && intFOB - intLstFOB >= 5 && Convert.ToInt32(aryTextValues[99]) < 5)
						sbytStartRF += 2;
					if ((sbytStartRF & 1) == 0 && boolFuelConverter && aryTextValues[90] == "1")
						sbytStartRF += 1;
					swRF.Start();
				}
				//Refuelling started
				if (sbytStartRF > 0)
				{
					//Check if fuel converter was fuel a source
					if ((sbytStartRF & 1) != 0)
					{
						// Check tractor beam is on, to calculate else cancel
						if (aryTextValues[90] == "1")
						{
							dblFillRate += 1.5;
							dblERR = 0.1 * (intTank - intFOB) / 60;
						}
						else
							sbytStartRF -= 1;
					}
					//Check if deployed fuel processor was a fuel source
					if (sbytStartRF > 1)
					{
						// Check velocity, to calculate values, cancel if it's too high
						if (Convert.ToInt32(aryTextValues[99]) < 5)
						{
							dblFillRate += (double)5 / (dblPeriod*0.75);
							dblERR = 0.1 * (intTank - intFOB) / 60;
						}
						else
							sbytStartRF -= 2;
					}
					//Check refuelling status, and display info
					if (sbytStartRF == 0)
					{
						swRF.Reset();
						txtLine3.Text = "";
						txtLine4.Text = "";
						txtLine5.Text = "";
						txtLine6.Text = "";
					}
					else
					{
						dblERR *= 0.5;
						dblETF = (double)(intTank - intFOB) / dblFillRate / 60 + dblERR;
						this.txtLine3.Text = String.Format("Elapsed Time: {0}:{1:00}:{2:00}", Math.Truncate(swRF.Elapsed.TotalHours), swRF.Elapsed.Minutes, swRF.Elapsed.Seconds);
						if (intFOB < intTank - 1)
						{
							this.txtLine4.Text = "Tank: " + Math.Truncate(((double)intFOB / intTank) * 100 + 0.5).ToString() + " % Full";
							this.txtLine5.Foreground = System.Windows.Media.Brushes.White;
							if ((sbytTxtLineDisplay & 1 << 2) == 0 || Math.Truncate(dblETF + 0.5) < 1.0)
							{
								this.txtLine5.Text = "Time to Full: " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(dblETF + 0.5)));
								if (this.txtLine5.Text.IndexOf("Less") == -1)
									this.txtLine5.Text += " +/- " + Math.Truncate(dblERR + 0.5).ToString() + " minutes";
							}
							else
								this.txtLine5.Text = "Time to Full: " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(dblETF - dblERR + 0.5))) + " - " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(dblETF + dblERR + 0.5))) + " minutes";
						}
						else
						{
							this.txtLine4.Text = "Tank: 100 % Full";
							this.txtLine5.Text = "";
						}
						if (intFOB > intLstFOB && ((sbytStartRF & 1<<1) != 0 || this.txtLine6.Text.Length > 12 || this.txtLine6.Text.IndexOf("Refuelling") == -1))
							this.txtLine6.Text = "Refuelling";
						else if (this.txtLine6.Text.Length > Math.Truncate(dblPeriod*1.5 + 12.5) || intFOB >= intTank-1)
							swRF.Stop(); // Stop elapsed time 3 seconds after fuel should have increased if there was no fuel increase (it's either full, game is paused or player died)
						else
							txtLine6.Text += " .";
					}
					if (intFOB >= intTank - 3)
					{
						//Add audio for fuel full sound
						if (chkSound.IsChecked == true)
							Console.Beep();
					}
				}
				else if (sbytStartRF == 0) // Not refuelling so travelling
				{
					if (aryTextValues[82].IndexOf('S') > -1)
					{
						//Pull the range from the string (values before the S)
						intRange = Convert.ToInt32(aryTextValues[82].Substring(0, aryTextValues[82].IndexOf('S')));
						//If there's more than 1 sector distance, set the flag
						if (intRange > 1)
							boolMoreThan1Sector = true;

						//If there are more than a single jump number of sectors to go, perform trip calculations
						if (intRange > intDriveRange)
						{
							//Set the text of the text block
							this.txtLine3.Text = "Range to Destination: " + intRange.ToString() + " sectors";
							//Only do the calculations if there's a jump drive
							if (intDriveRange > 0)
								CalculateTripValues(intRange, intFOB, intDriveRange, dblPeriod);
							if (Convert.ToInt32(aryTextValues[15]) < 100 && aryTextValues[86] == "2")
							{
								if (intLstPwr != Convert.ToInt32(aryTextValues[15]) || txtLine6.Text.IndexOf("Jumping in: ") == -1)
									this.txtLine6.Text = "Jumping in: " + Convert.ToString(Math.Truncate((100 - Convert.ToInt32(aryTextValues[15])) * dblPeriod / 100 + 0.5)) + " seconds";
								else if (Convert.ToDouble(this.txtLine6.Text.Substring(12,this.txtLine6.Text.Length - 20)) > 0)
									this.txtLine6.Text = "Jumping in: " + Convert.ToString(Convert.ToInt32(this.txtLine6.Text.Substring(12, this.txtLine6.Text.Length - 20)) - 1) + " seconds";
							}
							else
								this.txtLine6.Text = "";
						}
						else if (intRange <= intDriveRange && Convert.ToDouble(aryTextValues[94]) < 10000)
						{
							if (intLstRange[0] > intDriveRange)
							{
								CalculateTripValues(intLstRange[0], intFOB, intDriveRange, dblPeriod);
								this.txtLine6.Text = "Jumping: " + intRange.ToString() + " sectors";
							}
							else
							{
								//Set the text of the text blocks
								this.txtLine3.Text = "Range to Jump Point: " + intRange.ToString() + " sectors";
								this.txtLine4.Text = "";
								this.txtLine5.Text = "";

								if (Convert.ToInt32(aryTextValues[15]) < 100 && aryTextValues[86] == "2" && boolMoreThan1Sector)
								{
									if (intLstPwr != Convert.ToInt32(aryTextValues[15]) || txtLine6.Text.IndexOf("Jumping in: ") == -1)
										this.txtLine6.Text = "Jumping in: " + Convert.ToString(Math.Truncate((100 - Convert.ToInt32(aryTextValues[15])) * dblPeriod / 100 + 0.5)) + " seconds";
									else if (Convert.ToDouble(this.txtLine6.Text.Substring(12,this.txtLine6.Text.Length - 20)) > 0)
										this.txtLine6.Text = "Jumping in: " + Convert.ToString(Convert.ToInt32(this.txtLine6.Text.Substring(12,this.txtLine6.Text.Length - 20)) - 1) + " seconds";
								}
								else
									this.txtLine6.Text = "";
							}
						}
					}
					//If there are fewer than 2 sectors of distance, display the generic message
					if (boolMoreThan1Sector == false)
					{
						//Set the text of the text block
						this.txtLine3.Text = "";
						this.txtLine4.Text = "";
						this.txtLine5.Text = "";
						this.txtLine6.Text = "";
						//Add audio end-of-trip reminder
						if (chkSound.IsChecked == true)
							if (aryTextValues[86] == "2")
								Console.Beep();
					}
				}

				//Check to make sure the program isn't paused, if it isn't, update last values & call the thread to do it all over again
				if (boolRunning)
				{
					intLstFOB = intFOB;
					intLstPwr = Convert.ToInt32(aryTextValues[15]);
					if (aryTextValues[86] == "2" && intLstRange[0] != -1)
					{
						if (intRange > intDriveRange || intLstRange[1] != Convert.ToInt32(aryTextValues[11]) || intLstRange[2] != Convert.ToInt32(aryTextValues[12]) || intLstRange[3] != Convert.ToInt32(aryTextValues[13]))
						{
							intLstRange[0] = intRange;
							intLstRange[1] = Convert.ToInt32(aryTextValues[11]);	//CurrentSX
							intLstRange[2] = Convert.ToInt32(aryTextValues[12]);	//CurrentSY
							intLstRange[3] = Convert.ToInt32(aryTextValues[13]);	//CurrentSZ
						}
					}
					else
					{
						intLstRange[0] = intRange;
						intLstRange[1] = Convert.ToInt32(aryTextValues[11]);
						intLstRange[2] = Convert.ToInt32(aryTextValues[12]);
						intLstRange[3] = Convert.ToInt32(aryTextValues[13]);
					}
					bw.RunWorkerAsync();
				}
			}
			else
			{
				swRF.Reset();
				sbytStartRF = 0;
				intLstPwr = 0;
				intLstFOB = -1;
				intLstRange[0] = -1;

				//Set the text of the text block
				this.txtLine1.Text = "";
				this.txtLine2.Text = "";
				this.txtLine3.Text = "";
				this.txtLine4.Text = "";
				this.txtLine5.Text = "";
				this.txtLine6.Text = "";
			}
		}


		//This is the function that executes when you click the INIT button on the GUI
		private void btnInit_Click(object sender, RoutedEventArgs e)
		{
			//Disable the INIT button, if you let someone click it again it'll crash trying to recreate the same thread
			if (!boolRunning)
			{
				btnInit.IsEnabled = false;
				boolRunning = true;
				btnInit.Opacity = btnPause.Background.Opacity * 0.90;
				btnInit.Background.Opacity = 1;
				//Call the backgroundworker thread to action
				bw.RunWorkerAsync();
			}
		}

		//This is the function that executes when you click the PAUSE button on the GUI
		private void btnPause_Click(object sender, RoutedEventArgs e)
		{
			//Re-enable the INIT button (resume)
			btnInit.IsEnabled = true;
			btnInit.Background.Opacity = btnPause.Background.Opacity;
			btnInit.Opacity = 1;
			boolRunning = false;
		}

		//This is the function that executes when you click the EXIT button on the GUI
		private void btnExit_Click(object sender, RoutedEventArgs e)
		{
			WriteINIFile();
			//Stop the background thread
			boolRunning = false;
			//Recover background thread resources
			Dispose();
			//Close the program
			this.Close();
		}

		//This is the function that executes when you drag the window
		private void window_MoveWindow(object sender, RoutedEventArgs e)
		{
			//Reposition the window
			DragMove();
		}

		//This function increases the window opacity
		private void btnOpacityPlus_Click(object sender, RoutedEventArgs e)
		{
			if (this.chkOpaqueText.IsChecked == true)
				SetTransparency(this.TextOutline.Background.Opacity + 0.10);
			else
				SetTransparency(this.Opacity + 0.10);
		}

		//This function decreases the window opacity
		private void btnOpacityMinus_Click(object sender, RoutedEventArgs e)
		{
			if (this.chkOpaqueText.IsChecked == true)
				SetTransparency(this.TextOutline.Background.Opacity - 0.10);
			else
				SetTransparency(this.Opacity - 0.10);
		}

		//This is the function that executes when the Opaque Text checkbox is altered
		private void TransparencyModeChanged(object sender, RoutedEventArgs e)
		{
			if (this.chkOpaqueText.IsChecked == true)
				SetTransparency(this.Opacity);
			else
				SetTransparency(this.TextOutline.Background.Opacity);
		}

		//This function executes on Left Mouse Button Down in TextBlock's 4 & 5.
		private void TextBlockMLBDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
		{
			if (boolRunning == true && e.ClickCount == 2)
			{
				if (sender == this.txtLine4 && this.txtLine4.Text != "")
				{
					if ((sbytTxtLineDisplay & 1) == 0)
						sbytTxtLineDisplay += 1;
					else
						sbytTxtLineDisplay -= 1;
				}
				else if (sender == this.txtLine5 && this.txtLine5.Text != "")
				{
					if (sbytStartRF == 0)
					{
						if ((sbytTxtLineDisplay & 1 << 1) == 0)
							sbytTxtLineDisplay += 2;
						else
							sbytTxtLineDisplay -= 2;
					}
					else
					{
						if ((sbytTxtLineDisplay & 1 << 2) == 0)
							sbytTxtLineDisplay += 4;
						else
							sbytTxtLineDisplay -= 4;
					}
				}
			}
		}

		private void SetTransparency(double Transparency)
		{
			if (this.chkOpaqueText.IsChecked == true)
			{
				if (Transparency <= 1 && Transparency > 0)
				{
					this.TextOutline.Background.Opacity = Transparency;
					this.TitleOutline.Background.Opacity = Transparency;
					if (btnInit.IsEnabled)
						this.btnInit.Background.Opacity = Transparency;
					else
						this.btnInit.Opacity = Transparency * 0.90;
					this.btnPause.Background.Opacity = Transparency;
					this.btnExit.Background.Opacity = Transparency;
					this.btnOpacityPlus.Background.Opacity = Transparency;
					this.btnOpacityMinus.Background.Opacity = Transparency;

					this.TextOutline.BorderBrush.Opacity = Transparency;
					this.TitleOutline.BorderBrush.Opacity = Transparency;
					this.Opacity = 1;
				}
			}
			else
			{
				if (Transparency <= 1 && Transparency > 0.09)
					this.Opacity = Transparency;
				else if (Transparency < 0.10)
					this.Opacity = 0.10;
				this.TextOutline.Background.Opacity = 1;
				this.TitleOutline.Background.Opacity = 1;
				this.btnInit.Background.Opacity = 1;
				this.btnInit.Opacity = 1;
				this.btnPause.Background.Opacity = 1;
				this.btnExit.Background.Opacity = 1;
				this.btnOpacityPlus.Background.Opacity = 1;
				this.btnOpacityMinus.Background.Opacity = 1;
				this.TextOutline.BorderBrush.Opacity = 1;
				this.TitleOutline.BorderBrush.Opacity = 1;
			}
		}

		private void CalculateTripValues(int Dist, int FOB, int MaxRange, double Power)
		{
			int MinJumps = 0;
			int MaxJumps = 0;
			double JumpTime = 7.0;
			double TimeRange = 0.1;
			MinJumps = GetJumpNum(Dist, MaxRange);
			if (MaxRange == 10 || MaxRange == 3)
				MaxJumps = GetJumpNum(Dist, MaxRange - 1);
			else
				MaxJumps = MinJumps;
			//Determine Estimated Time Enroute
			double Eslop = (double)((MaxJumps * (Power + JumpTime + TimeRange) - Power) - (MinJumps * (Power + JumpTime) - Power)) / 120;
			double ETE =   (double)((MaxJumps * (Power + JumpTime + TimeRange) - Power) + (MinJumps * (Power + JumpTime) - Power)) / 120;
			//Determine fuel consumption
			double Fuel = 1.004213 * Dist;
			double Fslop = Fuel * .04311414;
			if (aryTextValues[83] == "0")	// Check IDS Engaged
				Fuel += 1.5 / MaxRange * Dist;

			//Set the text of the text block
			if ((sbytTxtLineDisplay & 1) == 0 || Math.Truncate(ETE + 0.5) < 1.0)
			{
				this.txtLine4.Text = "Time Enroute: " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(ETE + 0.5)));
				if (this.txtLine4.Text.IndexOf("Less") == -1)
					this.txtLine4.Text += " +/- " + Math.Truncate(Eslop + 0.5).ToString() + " minutes";
			}
			else
				this.txtLine4.Text = "Time Enroute: " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(ETE - Eslop + 0.5))) + " - " + DisplayTS(TimeSpan.FromMinutes(Math.Truncate(ETE + Eslop + 0.5))) + " minutes";
			//Set the text of the text block
			if (FOB > Fuel + Fslop)
				this.txtLine5.Foreground = System.Windows.Media.Brushes.White;
			else if (FOB < Fuel - Fslop)
				this.txtLine5.Foreground = System.Windows.Media.Brushes.Red;
			else
				this.txtLine5.Foreground = System.Windows.Media.Brushes.Yellow;
			if ((sbytTxtLineDisplay & 1 << 1) == 0)
				this.txtLine5.Text = "Fuel Consumption: " + Math.Truncate(Fuel + 0.5).ToString() + " +/- " + Math.Truncate(Fslop + 0.5).ToString() + " units";
			else
				this.txtLine5.Text = "Fuel Consumption: " + Math.Truncate(Fuel - Fslop + 0.5).ToString() + " - " + Math.Truncate(Fuel + Fslop + 0.5).ToString() + " units";
		}

		private string DisplayTS(TimeSpan value)
		{
			string result = "";
			if (Math.Truncate(value.TotalHours) > 0)
				result = String.Format("{0} hr{1} {2}",Math.Truncate(value.TotalHours),Math.Truncate(value.TotalHours)>1?"s":"", value.Minutes);
			else if (value.Minutes > 0)
				result = String.Format("{0}", value.Minutes);
			else // if (value.Seconds > 0)
				result = "Less than 1 minute";
			return result;
		}

		private int GetJumpNum(int Dist, int Range)
		{
			int Num = 0;
			Num = Dist / Range;
			if ((double)Dist / Range - Num > 0.0)
				Num += 1;
			return Num;
		}

		private void ReadINIFile()
		{
			INIFile ini = new INIFile(@".\TravelBuddy.ini");

			strSaveDataPath = ini.INIReadValue("Settings", "Path");
			if (strSaveDataPath == "")
				strSaveDataPath = @"C:\sw3dg\EvochronMercenary\savedata.txt";

			if (ini.INIReadValue("Settings", "Sound") == "")
				this.chkSound.IsChecked = true;
			else
				this.chkSound.IsChecked = Convert.ToBoolean(ini.INIReadValue("Settings", "Sound"));
			if (ini.INIReadValue("Settings", "OpaqueText") == "")
				this.chkOpaqueText.IsChecked = false;
			else
				this.chkOpaqueText.IsChecked = Convert.ToBoolean(ini.INIReadValue("Settings", "OpaqueText"));
			if (ini.INIReadValue("Settings", "RestoreWindow") == "")
				this.chkRestoreWindow.IsChecked = false;
			else
				this.chkRestoreWindow.IsChecked = Convert.ToBoolean(ini.INIReadValue("Settings", "RestoreWindow"));
			if (ini.INIReadValue("Window", "LineFormat") == "")
				sbytTxtLineDisplay = 0;
			else
				sbytTxtLineDisplay = Convert.ToSByte(ini.INIReadValue("Window", "LineFormat"));

			if (this.chkRestoreWindow.IsChecked == true)
			{
				if (ini.INIReadValue("Window", "PositionX") != "")
					this.Left = Convert.ToDouble(ini.INIReadValue("Window", "PositionX"));
				if (ini.INIReadValue("Window", "PositionY") != "")
					this.Top = Convert.ToDouble(ini.INIReadValue("Window", "PositionY"));
				if (ini.INIReadValue("Window", "Opacity") != "")
					SetTransparency(Convert.ToDouble(ini.INIReadValue("Window", "Opacity")));
			}
		}

		private void WriteINIFile()
		{
			INIFile ini = new INIFile(@".\TravelBuddy.ini");

			ini.INIWriteValue("Settings", "Path", strSaveDataPath);
			ini.INIWriteValue("Settings", "Sound", this.chkSound.IsChecked.ToString());
			ini.INIWriteValue("Settings", "OpaqueText", this.chkOpaqueText.IsChecked.ToString());
			ini.INIWriteValue("Settings", "RestoreWindow", chkRestoreWindow.IsChecked.ToString());
			ini.INIWriteValue("Window", "LineFormat", sbytTxtLineDisplay.ToString());
			if (this.chkRestoreWindow.IsChecked == true)
			{
				ini.INIWriteValue("Window", "PositionX", this.Left.ToString());
				ini.INIWriteValue("Window", "PositionY", this.Top.ToString());
				if (this.chkOpaqueText.IsChecked == false)
					ini.INIWriteValue("Window", "Opacity", this.Opacity.ToString());
				else
					ini.INIWriteValue("Window", "Opacity", this.TextOutline.Background.Opacity.ToString());
			}
		}

		public void Dispose()
		{
			bw.Dispose();
			GC.SuppressFinalize(bw);
		}

	}
}
